#!/bin/bash

# set sane defaults
IFS=$' \t\n'
export LANG=C
export LC_NUMERIC=C
export LC_TIME=C

die()
{
  [ "$1" == '0' ] && printf '%s\n' "$2" || printf '%s\n' "$2" 1>&2
  exit "$1"
}

syntax()
{
  die "$1" \
'Usage: op5 log merge [OPTION] [--] <LOG_TYPE>..

Available log types:
nagios (naemon), merlin-daemon, merlin-neb, merlin (daemon+neb)

Options:
-1, --start  Only include log files containing log data start from this date.
             The "date" can be any timestamp compatible with date(1).
-2, --end    Only include log files containing log data up until this date.
-c, --count  Print the number of files to be merged.
-l, --list   List the files to be merged.
-s, --size   Print the disk size of the files to be merged.
'
}


runcmd()
{
  if [ -n "$cmdpost" ]; then
    $cmd "$@" | $cmdpost
  else
    $cmd "$@"
  fi
}

add()
{
  local f fr
  for f; do
    fr="$(readlink -e -- "$f")"
    [ -n "$fr" ] || continue
    [ -n "${file["$fr"]}" ] && continue

    check_file_ts "$fr" || continue

    file["$fr"]=' '
  done
}

check_file_ts()
{
  local f_ts1 f_ts2
  f_ts1="$(head -n10 -- "$1" | grep -Po '(?<=^\[)[0-9]+(?=\])' | head -n1)"
  [ -n "$f_ts1" ] || return 1
  f_ts2="$(tail -n10 -- "$1" | grep -Po '(?<=^\[)[0-9]+(?=\])' | tail -n1)"
  [ -n "$f_ts2" ] || return 2

  # Ignore file if it begins with a more recent entry than our end time.
  [ "$f_ts1" -gt "$ts2" ] && return 3
  # Ignore file if it ends with an older entry than our start time.
  [ "$f_ts2" -lt "$ts1" ] && return 4

  return 0
}

lookup()
{
  case "$1" in
    nagios|naemon)
      add /opt/monitor/var/nagios.log \
        /opt/monitor/var/naemon.log \
        /opt/monitor/var/archives/*.log*
      ;;
    merlin-daemon)
      add /opt/monitor/op5/merlin/logs/daemon.log* \
        /var/log/op5/merlin/daemon.log*
      ;;
    merlin-neb)
      add /opt/monitor/op5/merlin/logs/neb.log* \
        /var/log/op5/merlin/neb.log*
      ;;
    merlin)
      lookup merlin-daemon
      lookup merlin-neb
      ;;
    all)
      lookup nagios
      lookup merlin
      ;;
    *)
      die '1' "Invalid log type: $1"
      ;;
  esac
}


get_ts()
{
  if [ "$1" -ge '0' ] &> /dev/null; then
    printf '%s\n' "$1"
  else
    date -d "$1" +%s || exit 1
  fi
}

cmd='zcat -f'
cmdpost=''

ts1=0 # Thu Jan  1 01:00:00 CET 1970
ts2=9999999999 # Sat Nov 20 18:46:39 CET 2286

while [ "$#" -gt '0' ]; do
  case "$1" in
    -h|--help)
      syntax 0
      ;;
    -1|--start)
      [ -n "$2" ] || die '1' "Missing argument to '$1'."
      ts1="$(get_ts "$2")" || exit 1
      shift 2
      ;;
    -2|--end)
      [ -n "$2" ] || die '1' "Missing argument to '$1'."
      ts2="$(get_ts "$2")" || exit 1
      shift 2
      ;;
    -c|--count)
      cmd='ls -1df'
      cmdpost='wc -l'
      shift
      ;;
    -l|--list)
      cmd='ls -1d'
      cmdpost='cat'
      shift
      ;;
    -s|--size)
      cmd='ls -1df'
      cmdpost='size'
      size() { xargs du -shc | tail -n1 | awk '{print $1}'; }
      shift
      ;;
    --)
      shift
      break
      ;;
    *)
      break
      ;;
  esac
done

[ "$ts1" -gt "$ts2" ] && \
  die '1' 'Start time cannot be later than end time.'


declare -A file
shopt -s nullglob

if [ -n "$1" ]; then
  for type; do
   lookup "$type"
  done
else
  lookup all
fi

runcmd "${!file[@]}"
